@@ -4,7 +4,6 @@
 #include "hashtable.h"
 #include "list.h"
 #include "util/stringops.h"
-#include "util/assert.h"
 #include "log.h"
 
 #include <SDL.h>
@@ -158,7 +157,7 @@
  *
  * Example:
  *
- *        #define HT_FUNC_COPY_KEY(dst, src) (*(dst) = strdup(src))
+ *        #define HT_FUNC_COPY_KEY(dst, src) (*(dst) = mem_strdup(src))
  */
 #ifndef HT_FUNC_COPY_KEY
 	#define HT_FUNC_COPY_KEY(dst, src) (*(dst) = (src))
