@@ -2,21 +2,29 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
-#include <stdio.h>
-
-#include "menu.h"
-#include "common.h"
 #include "options.h"
+
+#include "common.h"
+#include "config.h"
+#include "events.h"
 #include "mainmenu.h"
-#include "global.h"
-#include "video.h"
+#include "menu.h"
+
+#include "audio/audio.h"
+#include "gamepad.h"
+#include "renderer/api.h"
+#include "resource/font.h"
 #include "util/graphics.h"
+#include "video.h"
+
+#define OPTIONS_ACTIVE_X_OFFSET 20   /* FIXME hardcoded in draw_menu_list */
+#define OPTIONS_ITEM_HEIGHT     20   /* FIXME hardcoded in draw_menu_list */
+#define OPTIONS_X_MARGIN 100
+#define OPTIONS_Y_MARGIN 100
 
 typedef struct OptionBinding OptionBinding;
 
@@ -37,7 +45,22 @@ typedef enum BindingType {
 	BT_VideoDisplay,
 } BindingType;
 
+typedef enum OptionsMenuEntryArgType {
+	ARGTYPE_BIND = 0xafafaf00,
+	ARGTYPE_OTHER,
+} OptionsMenuEntryArgType;
+
+// Shared header for all structs used as args in menu entries,
+// because past me had the brilliant idea of storing the bindings in there
+// and now there is a need to tell them apart from other stuff.
+// One of these days I'm gonna rewrite this whole file from scratch (copium)
+// along with the entire menu system (mega copium), but that day is not today.
+typedef struct OptionsMenuArgHeader {
+	OptionsMenuEntryArgType type;
+} OptionsMenuArgHeader;
+
 typedef struct OptionBinding {
+	OptionsMenuArgHeader header;
 	union {
 		char **values;
 		char *strvalue;
@@ -62,6 +85,7 @@ typedef struct OptionBinding {
 
 static OptionBinding* bind_new(void) {
 	return ALLOC(OptionBinding, {
+		.header.type = ARGTYPE_BIND,
 		.selected = -1,
 		.configentry = -1,
 	});
@@ -83,7 +107,18 @@ static void bind_free(OptionBinding *bind) {
 
 static OptionBinding* bind_get(MenuData *m, int idx) {
 	MenuEntry *e = dynarray_get_ptr(&m->entries, idx);
-	return e->arg == m? NULL : e->arg;
+	OptionsMenuArgHeader *header = e->arg;
+
+	if(!header) {
+		return NULL;
+	}
+
+	if(header->type == ARGTYPE_BIND) {
+		return CASTPTR_ASSUME_ALIGNED(header, OptionBinding);
+	}
+
+	assume(header->type == ARGTYPE_OTHER);
+	return NULL;
 }
 
 // BT_IntValue: integer and boolean options
@@ -278,7 +313,7 @@ static int bind_addvalue(OptionBinding *b, char *val) {
 	}
 
 	b->values = mem_realloc(b->values, (1 + b->valrange_max) * sizeof(char*));
-	b->values[b->valrange_max] = strdup(val);
+	b->values[b->valrange_max] = mem_strdup(val);
 	return b->valrange_max;
 }
 
@@ -447,12 +482,36 @@ static int bind_gamepad_set(OptionBinding *b, int v) {
 
 // --- Creating, destroying, filling the menu --- //
 
+typedef struct ConfirmDialog {
+	OptionsMenuArgHeader header;
+	const char *const text;
+	void (*const action)(void);
+} ConfirmDialog;
+
+static const ConfirmDialog dialog_reset_default = {
+	.header.type = ARGTYPE_OTHER,
+	.text = "Reset all settings to defaults?",
+	.action = config_reset,
+};
+
+static const ConfirmDialog dialog_reset_saved = {
+	.header.type = ARGTYPE_OTHER,
+	.text = "Reset all settings to the last saved values?",
+	.action = config_load,
+};
+
 typedef struct OptionsMenuContext {
 	const char *title;
 	void *data;
 	MenuData *submenu;
 	MenuData *submenu_fading;
+	ConfirmDialog *confirm_dialog;
 	float submenu_alpha;
+	void (*draw_overlay)(MenuData *m, struct OptionsMenuContext *ctx);
+	struct {
+		bool allowed;
+		bool active;
+	} gamepad_testmode;
 } OptionsMenuContext;
 
 static void destroy_options_menu(MenuData *m) {
@@ -564,15 +623,13 @@ static void options_enter_sub(MenuData *parent, MenuData *(*construct)(MenuData*
 		options_enter_sub(parent, construct); \
 	}
 
-static void reset_to_defaults_draw(MenuData *m) {
+static void confirm_dialog_draw(MenuData *m) {
 	OptionsMenuContext *ctx = m->context;
 	float alpha = ctx->submenu_alpha;
 
-	static const char confirm_text[] = "Reset all settings to defaults?";
-
 	float lineskip = font_get_lineskip(res_font("standard"));
 	float height = lineskip * 4;
-	float width  = text_width(res_font("standard"), confirm_text, 0) + 64;
+	float width  = text_width(res_font("standard"), ctx->confirm_dialog->text, 0) + 64;
 
 	r_state_push();
 	alpha *= 0.7;
@@ -594,7 +651,7 @@ static void reset_to_defaults_draw(MenuData *m) {
 
 	r_state_pop();
 
-	text_draw(confirm_text, &(TextParams) {
+	text_draw(ctx->confirm_dialog->text, &(TextParams) {
 		.align = ALIGN_CENTER,
 		.color = RGBA_MUL_ALPHA(1, 1, 1, alpha),
 		.pos = { SCREEN_W*0.5, SCREEN_H*0.5 - lineskip * 0.5 },
@@ -624,18 +681,19 @@ static void reset_to_defaults_draw(MenuData *m) {
 }
 
 static void confirm_reset(MenuData *m, void *a) {
-	config_reset();
+	OptionsMenuContext *ctx = m->context;
+	ctx->confirm_dialog->action();
 	menu_action_close(m, a);
 }
 
-static void reset_to_defaults_logic(MenuData *m) {
+static void confirm_dialog_logic(MenuData *m) {
 	animate_menu_list_entries(m);
 }
 
-static void reset_to_defaults(MenuData *m, void *a) {
+static void confirm_dialog(MenuData *m, void *a) {
 	MenuData *sub = alloc_menu();
-	sub->draw = reset_to_defaults_draw;
-	sub->logic = reset_to_defaults_logic;
+	sub->draw = confirm_dialog_draw;
+	sub->logic = confirm_dialog_logic;
 	sub->flags = MF_Transient | MF_Abortable;
 	sub->transition = NULL;
 	sub->context = m->context;
@@ -647,6 +705,7 @@ static void reset_to_defaults(MenuData *m, void *a) {
 	assert(!ctx->submenu);
 	ctx->submenu = sub;
 	ctx->submenu_alpha = 0;
+	ctx->confirm_dialog = a;
 
 	if(ctx->submenu_fading) {
 		free_menu(ctx->submenu_fading);
@@ -832,12 +891,83 @@ static void destroy_options_menu_gamepad(MenuData *m) {
 	destroy_options_menu(m);
 }
 
+static inline GamepadButton options_gamepad_testing_button(void) {
+	return config_get_int(CONFIG_GAMEPAD_KEY_FOCUS);
+}
+
+static void draw_gamepad_options_overlay(MenuData *m, OptionsMenuContext *ctx) {
+	float csize = 86;
+
+	int x, y;
+	gamepad_get_player_analog_input(&x, &y);
+
+	cmplx in_processed = CMPLX(
+		gamepad_normalize_axis_value(x),
+		gamepad_normalize_axis_value(y));
+
+	cmplx in_raw = CMPLX(
+		gamepad_normalize_axis_value(gamepad_player_axis_value(PLRAXIS_LR)),
+		gamepad_normalize_axis_value(gamepad_player_axis_value(PLRAXIS_UD)));
+
+	double deadzone = gamepad_get_normalized_deadzone();
+	double maxzone = gamepad_get_normalized_maxzone();
+
+	r_mat_mv_push();
+	r_mat_mv_translate(
+		SCREEN_W - OPTIONS_X_MARGIN,
+		OPTIONS_Y_MARGIN + 6.5f*OPTIONS_ITEM_HEIGHT,
+	0);
+	r_mat_mv_scale(csize, csize, 0);
+	r_mat_mv_translate(0.5f, 0.5f, 0);
+	float snap = clamp(config_get_float(CONFIG_GAMEPAD_AXIS_SNAP), 0, 1);
+	float diagonal_bias = clamp(config_get_float(CONFIG_GAMEPAD_AXIS_SNAP_DIAG_BIAS), -1, 1);
+	float w_cardinal = 1.0f - diagonal_bias;
+	float w_diagonal = 1.0f + diagonal_bias;
+	float thres_card = (snap * (float)M_PI/8.0f) * w_cardinal;
+	float thres_diag = (snap * (float)M_PI/8.0f) * w_diagonal;
+	r_shader("gamepad_circle");
+	r_uniform_vec4("snap_angles_sincos", sinf(thres_card), cosf(thres_card), sinf(thres_diag), cosf(thres_diag));
+	r_uniform_vec2("deadzones", deadzone, maxzone);
+
+	if(ctx->gamepad_testmode.active) {
+		r_uniform_vec4("joy_pointers", re(in_processed), im(in_processed), re(in_raw), im(in_raw));
+		r_color4(1, 1, 1, 1);
+	} else {
+		r_uniform_vec4("joy_pointers", 69, 69, 69, 69);
+		r_color4(0.7, 0.7, 0.7, 0.7);
+	}
+
+	r_draw_quad();
+	r_mat_mv_pop();
+
+	char buf[128];
+	GamepadButton test_btn = options_gamepad_testing_button();
+
+	if(ctx->gamepad_testmode.active) {
+		snprintf(buf, sizeof(buf),
+			"Press any button on your gamepad to exit joystick testing mode");
+	} else {
+		snprintf(buf, sizeof(buf),
+			"Press %s on your gamepad to enter joystick testing mode",
+			gamepad_button_name(test_btn));
+	}
+
+	text_draw(buf, &(TextParams) {
+		.pos = { SCREEN_W/2.0f, SCREEN_H - OPTIONS_Y_MARGIN },
+		.align = ALIGN_CENTER,
+		.shader = "text_default",
+		.color = RGBA(0.7, 0.7, 0.7, 0.7),
+	});
+}
+
 static MenuData* create_options_menu_gamepad(MenuData *parent) {
 	MenuData *m = create_options_menu_base("Gamepad Options");
 	m->end = destroy_options_menu_gamepad;
 
 	OptionsMenuContext *ctx = m->context;
-	ctx->data = strdup(config_get_str(CONFIG_GAMEPAD_DEVICE));
+	ctx->data = mem_strdup(config_get_str(CONFIG_GAMEPAD_DEVICE));
+	ctx->draw_overlay = draw_gamepad_options_overlay;
+	ctx->gamepad_testmode.allowed = true;
 
 	OptionBinding *b;
 
@@ -856,15 +986,21 @@ static MenuData* create_options_menu_gamepad(MenuData *parent) {
 
 	add_menu_separator(m);
 
-	add_menu_entry(m, "Axes mode", do_nothing,
-		b = bind_option(CONFIG_GAMEPAD_AXIS_FREE, bind_common_onoff_get, bind_common_onoff_set)
-	);	bind_addvalue(b, "free");
-		bind_addvalue(b, "restricted");
-
 	add_menu_entry(m, "Remap square input into circular", do_nothing,
 		b = bind_option(CONFIG_GAMEPAD_AXIS_SQUARECIRCLE, bind_common_onoff_get, bind_gamepad_set)
 	);	bind_onoff(b);
 
+	add_menu_separator(m);
+
+	add_menu_entry(m, "Direction snap factor", do_nothing,
+		b = bind_scale(CONFIG_GAMEPAD_AXIS_SNAP, 0, 1, 0.05)
+	);
+
+	add_menu_entry(m, "Diagonal bias", do_nothing,
+		b = bind_scale(CONFIG_GAMEPAD_AXIS_SNAP_DIAG_BIAS, -1, 1, 0.05)
+	);	b->pad++;
+
+
 	add_menu_separator(m);
 
 	add_menu_entry(m, "Dead zone", do_nothing,
@@ -1061,7 +1197,11 @@ MenuData* create_options_menu(void) {
 	add_menu_entry(m, "Gamepad & Joystick options…", enter_options_menu_gamepad, NULL);
 	add_menu_separator(m);
 
-	auto e = add_menu_entry(m, "Reset to defaults", reset_to_defaults, NULL);
+	auto e = add_menu_entry(
+		m, "Reload from last saved", confirm_dialog, (void*)&dialog_reset_saved);
+	e->transition = NULL;
+	e = add_menu_entry(
+		m, "Reset to defaults", confirm_dialog, (void*)&dialog_reset_default);
 	e->transition = NULL;
 	add_menu_separator(m);
 
@@ -1089,7 +1229,7 @@ void draw_options_menu_bg(MenuData* menu) {
 static void update_options_menu(MenuData *menu) {
 	menu->drawdata[0] += ((SCREEN_W/2 - 100) - menu->drawdata[0])/10.0;
 	menu->drawdata[1] += ((SCREEN_W - 200) * 1.75 - menu->drawdata[1])/10.0;
-	menu->drawdata[2] += (20*menu->cursor - menu->drawdata[2])/10.0;
+	menu->drawdata[2] += (OPTIONS_ITEM_HEIGHT*menu->cursor - menu->drawdata[2])/10.0;
 
 	animate_menu_list_entries(menu);
 
@@ -1134,12 +1274,13 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 	}
 
 	text_draw(e->name, &(TextParams) {
-		.pos = { (1 + (bind ? bind->pad : 0)) * 20 - e->drawdata, 20*i },
+		.pos = { (1 + (bind ? bind->pad : 0)) * OPTIONS_ACTIVE_X_OFFSET - e->drawdata, OPTIONS_ITEM_HEIGHT*i },
 		.color = &clr,
 	});
 
 	if(bind) {
-		int j, origin = SCREEN_W - 220;
+		float margin = OPTIONS_X_MARGIN * 2 + OPTIONS_ACTIVE_X_OFFSET;
+		float origin = SCREEN_W - margin;
 
 		switch(bind->type) {
 			case BT_IntValue: {
@@ -1153,7 +1294,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 						val,
 						ALIGN_RIGHT,
 						origin,
-						20*i,
+						OPTIONS_ITEM_HEIGHT*i,
 						fnt_int,
 						fnt_fract,
 						&clr,
@@ -1161,7 +1302,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 						false
 					);
 				} else if(bind->values) {
-					for(j = bind->displaysingle? val : bind->valrange_max; (j+1) && (!bind->displaysingle || j == val); --j) {
+					for(int j = bind->displaysingle? val : bind->valrange_max; (j+1) && (!bind->displaysingle || j == val); --j) {
 						if(j != bind->valrange_max && !bind->displaysingle) {
 							origin -= text_width(res_font("standard"), bind->values[j+1], 0) + 5;
 						}
@@ -1173,7 +1314,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 						}
 
 						text_draw(bind->values[j], &(TextParams) {
-							.pos = { origin, 20*i },
+							.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 							.align = ALIGN_RIGHT,
 							.color = &clr,
 						});
@@ -1194,7 +1335,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 			case BT_KeyBinding: {
 				if(bind->blockinput) {
 					text_draw("Press a key to assign, ESC to cancel", &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = RGBA(0.5, 1, 0.5, 1),
 					});
@@ -1206,7 +1347,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 					}
 
 					text_draw(txt, &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = &clr,
 					});
@@ -1243,10 +1384,10 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 					}
 
 					text_draw(txt, &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = &clr,
-						.max_width = (SCREEN_W - 220) / 2,
+						.max_width = (SCREEN_W - margin) / 2,
 					});
 				}
 
@@ -1268,10 +1409,10 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 				char buf[64];
 				snprintf(buf, sizeof(buf), "#%i: %s", bind->selected + 1, video_display_name(bind->selected));
 				text_draw(buf, &(TextParams) {
-					.pos = { origin, 20*i },
+					.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 					.align = ALIGN_RIGHT,
 					.color = &clr,
-					.max_width = (SCREEN_W - 220) / 2,
+					.max_width = (SCREEN_W - margin) / 2,
 				});
 				break;
 			}
@@ -1285,7 +1426,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 											: "Press a button to assign, Back to cancel";
 
 					text_draw(text, &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = RGBA(0.5, 1, 0.5, 1),
 					});
@@ -1293,13 +1434,13 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 					int id = config_get_int(bind->configentry);
 					const char *name = (is_axis ? gamepad_axis_name(id) : gamepad_button_name(id));
 					text_draw(name, &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = &clr,
 					});
 				} else {
 					text_draw("Unbound", &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = &clr,
 					});
@@ -1311,14 +1452,14 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 				if(bind->blockinput) {
 					if(*bind->strvalue) {
 						text_draw(bind->strvalue, &(TextParams) {
-							.pos = { origin, 20*i },
+							.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 							.align = ALIGN_RIGHT,
 							.color = RGBA(0.5, 1, 0.5, 1.0),
 						});
 					}
 				} else {
 					text_draw(config_get_str(bind->configentry), &(TextParams) {
-						.pos = { origin, 20*i },
+						.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 						.align = ALIGN_RIGHT,
 						.color = &clr,
 					});
@@ -1340,7 +1481,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 
 				snprintf(tmp, sizeof(tmp), "%dx%d", w, h);
 				text_draw(tmp, &(TextParams) {
-					.pos = { origin, 20*i },
+					.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 					.align = ALIGN_RIGHT,
 					.color = &clr,
 				});
@@ -1352,7 +1493,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 				char tmp[32];
 				snprintf(tmp, sizeof(tmp), "%gx  %dx%d", video_get_scaling_factor(), fbsize.w, fbsize.h);
 				text_draw(tmp, &(TextParams) {
-					.pos = { origin, 20*i },
+					.pos = { origin, OPTIONS_ITEM_HEIGHT*i },
 					.align = ALIGN_RIGHT,
 					.color = &clr,
 				});
@@ -1376,7 +1517,7 @@ static void options_draw_item(MenuEntry *e, int i, int cnt, void *ctx) {
 					strcpy(tmp, "0%");
 
 				r_mat_mv_push();
-				r_mat_mv_translate(origin - (w+cw) * 0.5, 20 * i, 0);
+				r_mat_mv_translate(origin - (w+cw) * 0.5, OPTIONS_ITEM_HEIGHT * i, 0);
 				text_draw(tmp, &(TextParams) {
 					.pos = { -((w+cw) * 0.5 + 10), 0 },
 					.align = ALIGN_RIGHT,
@@ -1406,9 +1547,15 @@ static void draw_options_menu(MenuData *menu) {
 	r_state_push();
 	draw_options_menu_bg(menu);
 	draw_menu_title(menu, ctx->title);
-	draw_menu_list(menu, 100, 100, options_draw_item, SCREEN_H * 1.1, menu);
+	draw_menu_list(menu, OPTIONS_X_MARGIN, OPTIONS_Y_MARGIN, options_draw_item, SCREEN_H * 1.1, menu);
 	r_state_pop();
 
+	if(ctx->draw_overlay) {
+		r_state_push();
+		ctx->draw_overlay(menu, ctx);
+		r_state_pop();
+	}
+
 	if(ctx->submenu_fading) {
 		r_state_push();
 		ctx->submenu_fading->draw(ctx->submenu_fading);
@@ -1656,9 +1803,21 @@ static bool options_text_input_handler(SDL_Event *event, void *arg) {
 
 static bool options_input_handler(SDL_Event *event, void *arg) {
 	MenuData *menu = arg;
+	OptionsMenuContext *ctx = menu->context;
 	OptionBinding *bind = bind_get(menu, menu->cursor);
 	TaiseiEvent type = TAISEI_EVENT(event->type);
 
+	switch(type) {
+		case TE_MENU_CURSOR_UP:
+		case TE_MENU_CURSOR_DOWN:
+		case TE_MENU_CURSOR_LEFT:
+		case TE_MENU_CURSOR_RIGHT:
+			if((intptr_t)event->user.data1 == INDEV_GAMEPAD && ctx->gamepad_testmode.active) {
+				return false;
+			}
+		default: break;
+	}
+
 	switch(type) {
 		case TE_MENU_CURSOR_UP:
 		case TE_MENU_CURSOR_DOWN:
@@ -1733,6 +1892,26 @@ static bool options_input_handler(SDL_Event *event, void *arg) {
 			close_menu(menu);
 		break;
 
+		case TE_GAMEPAD_BUTTON_DOWN:
+			if(ctx->gamepad_testmode.allowed) {
+				if(ctx->gamepad_testmode.active) {
+					switch(event->user.code) {
+						case GAMEPAD_BUTTON_ANALOG_STICK_DOWN:
+						case GAMEPAD_BUTTON_ANALOG_STICK_LEFT:
+						case GAMEPAD_BUTTON_ANALOG_STICK_RIGHT:
+						case GAMEPAD_BUTTON_ANALOG_STICK_UP:
+							break;
+						default:
+							ctx->gamepad_testmode.active = false;
+					}
+				} else {
+					if(event->user.code == options_gamepad_testing_button()) {
+						ctx->gamepad_testmode.active = true;
+					}
+				}
+			}
+		break;
+
 		default: break;
 	}
 
