@@ -2,20 +2,19 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
+#include "progress.h"
+
+#include "stageinfo.h"
+#include "util.h"
+#include "version.h"
+#include "rwops/rwops_zstd.h"
 
 #include <zlib.h>
 
-#include "progress.h"
-#include "stageinfo.h"
-#include "version.h"
-#include "rwops/rwops_autobuf.h"
-#include "rwops/rwops_zstd.h"
-
 /*
 	This module implements a persistent storage of a player's game progress, such as unlocked stages, high-scores etc.
 
