@@ -2,17 +2,16 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
 #pragma once
 #include "taisei.h"
 
-#include "util.h"
-#include "pixmap/pixmap.h"
 #include "color.h"
-#include "common/shaderlib/shaderlib.h"
+#include "pixmap/pixmap.h"
+#include "renderer/common/shaderlib/shaderlib.h"
 #include "resource/resource.h"
 #include "resource/shader_program.h"
 #include "resource/texture.h"
@@ -124,6 +123,8 @@ typedef enum TextureClass {
 	// NOTE: whichever is placed first here is considered the "default" where applicable.
 	TEXTURE_CLASS_2D,
 	TEXTURE_CLASS_CUBEMAP,
+
+	NUM_TEXTURE_CLASSES,
 } TextureClass;
 
 typedef enum TextureFilterMode {
@@ -218,15 +219,24 @@ enum {
 	FRAMEBUFFER_MAX_OUTPUTS = FRAMEBUFFER_MAX_COLOR_ATTACHMENTS,
 };
 
+typedef void (*FramebufferReadAsyncCallback)(const Pixmap *pixmap, void *userdata);
+
 typedef enum Primitive {
 	PRIM_POINTS,
 	PRIM_LINE_STRIP,
-	PRIM_LINE_LOOP,
 	PRIM_LINES,
 	PRIM_TRIANGLE_STRIP,
 	PRIM_TRIANGLES,
 } Primitive;
 
+struct Model {
+	VertexArray *vertex_array;
+	size_t num_vertices;
+	size_t num_indices;
+	size_t offset;
+	Primitive primitive;
+};
+
 typedef enum VertexAttribType {
 	VA_FLOAT,
 	VA_BYTE,
@@ -430,10 +440,10 @@ typedef union ShaderCustomParams {
 } ShaderCustomParams;
 
 typedef struct SpriteStateParams {
+	ShaderProgram *shader;
 	Texture *primary_texture;
 	Texture *aux_textures[R_NUM_SPRITE_AUX_TEXTURES];
 	BlendMode blend;
-	ShaderProgram *shader;
 } SpriteStateParams;
 
 typedef union SpriteScaleParams {
@@ -455,17 +465,13 @@ typedef struct SpriteRotationParams {
 } SpriteRotationParams;
 
 typedef struct SpriteFlipParams {
-	unsigned char x : 1;
-	unsigned char y : 1;
+	bool x;
+	bool y;
 } SpriteFlipParams;
 
 typedef struct SpriteParams {
-	const char *sprite;
 	Sprite *sprite_ptr;
-
-	const char *shader;
 	ShaderProgram *shader_ptr;
-
 	Texture *aux_textures[R_NUM_SPRITE_AUX_TEXTURES];
 
 	// TODO: maybe embed these by value and get rid of SpriteParamsBuffer?
@@ -520,6 +526,7 @@ void r_post_init(void);
 void r_release_resources(void);
 void r_shutdown(void);
 const char *r_backend_name(void);
+void r_unclaim_window(SDL_Window *window);
 
 r_feature_bits_t r_features(void);
 
@@ -772,6 +779,8 @@ void r_framebuffer_destroy(Framebuffer *fb) attr_nonnull(1);
 void r_framebuffer_clear(Framebuffer *fb, BufferKindFlags flags, const Color *colorval, float depthval);
 void r_framebuffer_copy(Framebuffer *dst, Framebuffer *src, BufferKindFlags flags) attr_nonnull_all;
 IntExtent r_framebuffer_get_size(Framebuffer *fb);
+void r_framebuffer_read_async(Framebuffer *framebuffer, FramebufferAttachment attachment, IntRect region, void *userdata, FramebufferReadAsyncCallback callback);
+void r_framebuffer_read_viewport_async(Framebuffer *framebuffer, FramebufferAttachment attachment, void *userdata, FramebufferReadAsyncCallback callback);
 
 void r_framebuffer(Framebuffer *fb);
 Framebuffer* r_framebuffer_current(void);
@@ -818,8 +827,6 @@ VsyncMode r_vsync_current(void);
 
 void r_swap(SDL_Window *window);
 
-bool r_screenshot(Pixmap *dest) attr_nodiscard attr_nonnull(1);
-
 void r_mat_mv_push(void);
 void r_mat_mv_push_premade(mat4 mat);
 void r_mat_mv_push_identity(void);
