@@ -2,14 +2,15 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "backend.h"
 
+#include "util.h"
+#include "util/env.h"
+
 #undef R
 #define R(x) extern RendererBackend _r_backend_##x;
 TAISEI_BUILDCONF_RENDERER_BACKENDS
