@@ -2,31 +2,29 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
 #include "gl33.h"
+
 #include "../api.h"
-#include "../common/matstack.h"
 #include "../common/backend.h"
-#include "../common/sprite_batch.h"
-#include "texture.h"
-#include "shader_object.h"
-#include "shader_program.h"
-#include "framebuffer.h"
-#include "common_buffer.h"
-#include "vertex_buffer.h"
-#include "index_buffer.h"
-#include "vertex_array.h"
+#include "../common/matstack.h"
+#include "../common/sprite_batch_internal.h"
 #include "../glcommon/debug.h"
 #include "../glcommon/vtable.h"
-#include "resource/resource.h"
-#include "resource/model.h"
-#include "util/glm.h"
+#include "common_buffer.h"
+#include "framebuffer_async_read.h"
+#include "framebuffer.h"
+#include "index_buffer.h"
+#include "shader_object.h"
+#include "shader_program.h"
+#include "texture.h"
 #include "util/env.h"
+#include "util/glm.h"
+#include "vertex_array.h"
+#include "vertex_buffer.h"
 
 // #define GL33_DEBUG_TEXUNITS
 // #define GL33_DRAW_STATS
@@ -435,8 +433,6 @@ static void gl33_sync_viewport(void) {
 
 void gl33_sync_scissor(void) {
 	bool pending_enabled =
-		R.scissor.pending.x &&
-		R.scissor.pending.y &&
 		R.scissor.pending.w &&
 		R.scissor.pending.h;
 
@@ -574,7 +570,6 @@ GLenum gl33_prim_to_gl_prim(Primitive prim) {
 	static GLenum map[] = {
 		[PRIM_POINTS]         = GL_POINTS,
 		[PRIM_LINE_STRIP]     = GL_LINE_STRIP,
-		[PRIM_LINE_LOOP]      = GL_LINE_LOOP,
 		[PRIM_LINES]          = GL_LINES,
 		[PRIM_TRIANGLE_STRIP] = GL_TRIANGLE_STRIP,
 		[PRIM_TRIANGLES]      = GL_TRIANGLES,
@@ -838,6 +833,7 @@ GLenum gl33_bindidx_to_glenum(BufferBindingIndex bindidx) {
 		[GL33_BUFFER_BINDING_ARRAY] = GL_ARRAY_BUFFER,
 		[GL33_BUFFER_BINDING_COPY_WRITE] = GL_COPY_WRITE_BUFFER,
 		[GL33_BUFFER_BINDING_PIXEL_UNPACK] = GL_PIXEL_UNPACK_BUFFER,
+		[GL33_BUFFER_BINDING_PIXEL_PACK] = GL_PIXEL_PACK_BUFFER,
 	};
 
 	static_assert(sizeof(map) == sizeof(GLenum) * GL33_NUM_BUFFER_BINDINGS, "Fix the lookup table");
@@ -979,14 +975,12 @@ uint gl33_bind_texture(Texture *texture, GLuint lock_target, int preferred_unit)
 				(unit->pending != NULL || unit->locked_for_target != lock_target)
 			) {
 				// someone else already took this unit for rendering
-				unit = NULL;
+				log_fatal("BUG: can't bind %s to unit %i: unit already locked for rendering!",
+					texture->debug_label, preferred_unit);
 			}
 		}
 
-		if(unit == NULL) {
-			assert(!glext.issues.avoid_sampler_uniform_updates);
-			unit = gl33_get_free_texunit();
-		}
+		assume(unit != NULL);
 
 		if(unit->locked_for_target) {
 			gl33_relocate_texuint(unit);
@@ -1000,7 +994,7 @@ uint gl33_bind_texture(Texture *texture, GLuint lock_target, int preferred_unit)
 
 	assert(!lock_target || lock_target == texture->bind_target);
 
-	if(glext.issues.avoid_sampler_uniform_updates && preferred_unit >= 0) {
+	if(preferred_unit >= 0) {
 		assert(preferred_unit < R.texunits.limit);
 		TextureUnit *u = &R.texunits.array[preferred_unit];
 
@@ -1116,7 +1110,6 @@ void gl33_shader_deleted(ShaderProgram *prog) {
 
 	if(R.progs.pending == prog) {
 		R.progs.pending = NULL;
-		r_shader_standard();
 	}
 
 	if(R.progs.gl_prog == prog->gl_handle) {
@@ -1189,6 +1182,7 @@ static void gl33_post_init(void) {
 }
 
 static void gl33_shutdown(void) {
+	gl33_framebuffer_finalize_read_requests();
 	glcommon_unload_library();
 	SDL_GL_DeleteContext(R.gl_context);
 }
@@ -1391,6 +1385,7 @@ static void gl33_swap(SDL_Window *window) {
 #endif
 	r_framebuffer(prev_fb);
 
+	gl33_framebuffer_process_read_requests();
 	gl33_stats_post_frame();
 
 	// We can't rely on viewport being preserved across frames,
@@ -1424,18 +1419,6 @@ static DepthTestFunc gl33_depth_func_current(void) {
 	return R.depth_test.func.pending;
 }
 
-static bool gl33_screenshot(Pixmap *out) {
-	FloatRect *vp = &R.viewport.default_framebuffer;
-	out->width = vp->w;
-	out->height = vp->h;
-	out->format = PIXMAP_FORMAT_RGB8;
-	out->origin = PIXMAP_ORIGIN_BOTTOMLEFT;
-	out->data.untyped = pixmap_alloc_buffer_for_copy(out, &out->data_size);
-	glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
-	glReadPixels(vp->x, vp->y, vp->w, vp->h, GL_RGB, GL_UNSIGNED_BYTE, out->data.untyped);
-	return true;
-}
-
 static void gl33_scissor(IntRect scissor) {
 	R.scissor.pending = scissor;
 }
@@ -1509,6 +1492,7 @@ RendererBackend _r_backend_gl33 = {
 		.framebuffer_clear = gl33_framebuffer_clear,
 		.framebuffer_copy = gl33_framebuffer_copy,
 		.framebuffer_get_size = gl33_framebuffer_get_size,
+		.framebuffer_read_async = gl33_framebuffer_read_async,
 		.vertex_buffer_create = gl33_vertex_buffer_create,
 		.vertex_buffer_set_debug_label = gl33_vertex_buffer_set_debug_label,
 		.vertex_buffer_get_debug_label = gl33_vertex_buffer_get_debug_label,
@@ -1539,7 +1523,6 @@ RendererBackend _r_backend_gl33 = {
 		.vsync = gl33_vsync,
 		.vsync_current = gl33_vsync_current,
 		.swap = gl33_swap,
-		.screenshot = gl33_screenshot,
 	},
 	.custom = &(GLBackendData) {
 		.vtable = {
