@@ -2,18 +2,18 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
-#include "gl33.h"
 #include "shader_program.h"
-#include "shader_object.h"
-#include "texture.h"
-#include "../glcommon/debug.h"
+#include "shader_object.h"  // IWYU pragma: keep
+
 #include "../api.h"
+#include "../glcommon/debug.h"
+#include "gl33.h"
+#include "texture.h"
+#include "util.h"
 
 static Uniform *sampler_uniforms;
 
@@ -152,23 +152,6 @@ static struct {
 	[UNIFORM_MAT4]         = { uset_mat4,  uget_mat4 },
 };
 
-typedef struct MagicUniformSpec {
-	const char *name;
-	const char *typename;
-	UniformType type;
-} MagicUniformSpec;
-
-static MagicUniformSpec magic_unfiroms[] = {
-	[UMAGIC_MATRIX_MV]       = { "r_modelViewMatrix",     "mat4", UNIFORM_MAT4 },
-	[UMAGIC_MATRIX_PROJ]     = { "r_projectionMatrix",    "mat4", UNIFORM_MAT4 },
-	[UMAGIC_MATRIX_TEX]      = { "r_textureMatrix",       "mat4", UNIFORM_MAT4 },
-	[UMAGIC_COLOR]           = { "r_color",               "vec4", UNIFORM_VEC4 },
-	[UMAGIC_VIEWPORT]        = { "r_viewport",            "vec4", UNIFORM_VEC4 },
-	[UMAGIC_COLOR_OUT_SIZES] = { "r_colorOutputSizes[0]", "vec2", UNIFORM_VEC2 },
-	[UMAGIC_DEPTH_OUT_SIZE]  = { "r_depthOutputSize",     "vec2", UNIFORM_VEC2 },
-};
-static_assert(ARRAY_SIZE(magic_unfiroms) == NUM_MAGIC_UNIFORMS);
-
 static void gl33_update_uniform(Uniform *uniform, uint offset, uint count, const void *data) {
 	// these are validated properly in gl33_uniform
 	assert(offset < uniform->array_size);
@@ -235,30 +218,16 @@ static void *gl33_sync_uniform(const char *key, void *value, void *arg) {
 	// special case: for sampler uniforms, we have to construct the actual data from the texture pointers array.
 	UniformType utype = uniform->type;
 	if(UNIFORM_TYPE_IS_SAMPLER(utype)) {
-		Uniform *size_uniform = uniform->size_uniform;
-
 		for(uint i = 0; i < uniform->array_size; ++i) {
 			Texture *tex = uniform->textures[i];
 			GLuint preferred_unit = CASTPTR_ASSUME_ALIGNED(uniform->cache.pending, int)[i];
 			GLuint unit = gl33_bind_texture(tex, get_texture_target(tex, utype), preferred_unit);
 
+			assert(unit == preferred_unit);
+
 			if(unit != preferred_unit) {
 				gl33_update_uniform(uniform, i, 1, &unit);
 			}
-
-			if(size_uniform) {
-				uint w, h;
-
-				if(tex) {
-					r_texture_get_size(tex, 0, &w, &h);
-				} else {
-					w = h = 0;
-				}
-
-				vec2_noalign size = { w, h };
-				gl33_update_uniform(size_uniform, i, 1, &size);
-				gl33_commit_uniform(size_uniform);
-			}
 		}
 	}
 
@@ -326,6 +295,10 @@ static bool cache_uniforms(ShaderProgram *prog) {
 		glGetActiveUniform(prog->gl_handle, i, maxlen, NULL, &size, &type, name);
 		loc = glGetUniformLocation(prog->gl_handle, name);
 
+		if(strendswith(name, "[0]")) {
+			name[strlen(name) - 3] = 0;
+		}
+
 		if(loc < 0) {
 			// builtin uniform
 			continue;
@@ -404,19 +377,17 @@ static bool cache_uniforms(ShaderProgram *prog) {
 		if(UNIFORM_TYPE_IS_SAMPLER(uni.type)) {
 			list_push(&sampler_uniforms, new_uni);
 
-			if(glext.issues.avoid_sampler_uniform_updates) {
-				// Bind each sampler to a different texturing unit.
-				// This way we can change textures by binding them to the right texturing units, and never update the shader's samplers again.
+			// Bind each sampler to a different texturing unit.
+			// This way we can change textures by binding them to the right texturing units, and never update the shader's samplers again.
 
-				int payload[new_uni->array_size];
-				assert(sizeof(payload) == new_uni->array_size * new_uni->elem_size);
+			int payload[new_uni->array_size];
+			assert(sizeof(payload) == new_uni->array_size * new_uni->elem_size);
 
-				for(int j = 0; j < ARRAY_SIZE(payload); ++j) {
-					payload[j] = sampler_binding++;
-				}
-
-				gl33_update_uniform(new_uni, 0, new_uni->array_size, payload);
+			for(int j = 0; j < ARRAY_SIZE(payload); ++j) {
+				payload[j] = sampler_binding++;
 			}
+
+			gl33_update_uniform(new_uni, 0, new_uni->array_size, payload);
 		}
 
 		if(magic_index != UMAGIC_INVALID) {
@@ -429,39 +400,6 @@ static bool cache_uniforms(ShaderProgram *prog) {
 		log_debug("%s = %i [array elements: %i; size: %zi bytes]", name, loc, uni.array_size, uni.array_size * uni.elem_size);
 	}
 
-	ht_str2ptr_iter_t iter;
-	ht_iter_begin(&prog->uniforms, &iter);
-	while(iter.has_data) {
-		Uniform *u = iter.value;
-
-		if(UNIFORM_TYPE_IS_SAMPLER(u->type)) {
-			const char *sampler_name = iter.key;
-			const char size_suffix[] = "_SIZE";
-			char size_uniform_name[strlen(sampler_name) + sizeof(size_suffix)];
-			snprintf(size_uniform_name, sizeof(size_uniform_name), "%s%s", sampler_name, size_suffix);
-			u->size_uniform = ht_get(&prog->uniforms, size_uniform_name, NULL);
-
-			if(u->size_uniform) {
-				Uniform *size_uniform = u->size_uniform;
-
-				if(size_uniform->type != UNIFORM_VEC2) {
-					log_warn("Size uniform %s has invalid type (should be vec2), ignoring", size_uniform_name);
-					u->size_uniform = NULL;
-				} else if(size_uniform->array_size != u->array_size) {
-					log_warn("Size uniform %s has invalid array size (should be %i), ignoring", size_uniform_name, u->array_size);
-					u->size_uniform = NULL;
-				} else {
-					log_debug("Bound size uniform: %s --> %s", sampler_name, size_uniform_name);
-				}
-
-				assert(u->size_uniform != NULL);  // fix your shader!
-			}
-		}
-
-		ht_iter_next(&iter);
-	}
-	ht_iter_end(&iter);
-
 	return true;
 }
 
@@ -635,7 +573,6 @@ bool gl33_shader_program_transfer(ShaderProgram *dst, ShaderProgram *src) {
 			uold->array_size = unew->array_size;
 			uold->location = unew->location;
 			assert(uold->type == unew->type);
-			uold->size_uniform = ht_get(&old_new_map, unew->size_uniform, unew->size_uniform);
 			uold->cache = unew->cache;
 
 			if(UNIFORM_TYPE_IS_SAMPLER(unew->type)) {
@@ -648,7 +585,6 @@ bool gl33_shader_program_transfer(ShaderProgram *dst, ShaderProgram *src) {
 			// Deactivate, but keep the object around, because user code may be referencing it.
 			// We also need to keep type information, in case the uniform gets re-introduced.
 			uold->location = INVALID_UNIFORM_LOCATION;
-			uold->size_uniform = NULL;
 			uold->array_size = 0;
 			uold->textures = NULL;
 			uold->cache.pending = NULL;
@@ -668,7 +604,6 @@ bool gl33_shader_program_transfer(ShaderProgram *dst, ShaderProgram *src) {
 		assert(ht_get(&dst->uniforms, iter.key, NULL) == NULL);
 
 		unew->prog = dst;
-		unew->size_uniform = ht_get(&old_new_map, unew->size_uniform, unew->size_uniform);
 		ht_set(&dst->uniforms, iter.key, unew);
 	}
 
