@@ -2,18 +2,18 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
-#include "util.h"
-#include "rwops/rwops_autobuf.h"
 #include "opengl.h"
+
 #include "debug.h"
+#include "rwops/rwops_autobuf.h"
 #include "shaders.h"
 #include "texture.h"
+#include "util/env.h"
+#include "util/io.h"
 
 struct glext_s glext = { 0 };
 
@@ -180,20 +180,6 @@ ext_flag_t glcommon_check_extension(const char *ext) {
 	log_warn("Extension not supported"); \
 } while(0)
 
-ext_flag_t glcommon_require_extension(const char *ext) {
-	ext_flag_t val = glcommon_check_extension(ext);
-
-	if(!val) {
-		if(env_get("TAISEI_GL_REQUIRE_EXTENSION_FATAL", 0)) {
-			log_fatal("Required extension %s is not available", ext);
-		}
-
-		log_error("Required extension %s is not available, expect crashes or rendering errors", ext);
-	}
-
-	return val;
-}
-
 static void glcommon_ext_debug_output(void) {
 	EXT_FLAG(debug_output);
 
@@ -650,6 +636,25 @@ static void glcommon_ext_texture_format_fxt1(void) {
 	EXT_MISSING();
 }
 
+static void glcommon_ext_texture_storage(void) {
+	EXT_FLAG(texture_storage);
+
+	CHECK_CORE(GL_ATLEAST(4, 2));
+	CHECK_CORE(GLES_ATLEAST(3, 0));
+	CHECK_EXT(GL_ARB_texture_storage);
+
+	EXT_MISSING();
+}
+
+static void glcommon_ext_invalidate_subdata(void) {
+	EXT_FLAG(invalidate_subdata);
+
+	CHECK_CORE(GL_ATLEAST(4, 3));
+	CHECK_EXT(GL_ARB_invalidate_subdata);
+
+	EXT_MISSING();
+}
+
 static const char *get_unmasked_property(GLenum prop, bool fallback) {
 	const char *val = NULL;
 
@@ -747,7 +752,6 @@ static void detect_broken_intel_driver(void) {
 
 static bool glcommon_check_workaround(const char *name, const char *envvar, const char *(*detect)(void)) {
 	int env_setting = env_get_int(envvar, -1);
-	glext.issues.avoid_sampler_uniform_updates = false;
 
 	if(env_setting > 0) {
 		log_warn("Enabled workaround `%s` (forced by environment)", name);
@@ -781,26 +785,6 @@ EM_JS(bool, webgl_is_mac, (void), {
 })
 #endif
 
-static const char *detect_slow_sampler_update(void) {
-#if defined(__MACOSX__) || defined(__EMSCRIPTEN__)
-	const char *gl_vendor = get_unmasked_property(GL_VENDOR, true);
-	const char *gl_renderer = get_unmasked_property(GL_RENDERER, true);
-
-	if(
-#if defined(__EMSCRIPTEN__)
-		webgl_is_mac() &&
-#endif
-		strstr(gl_renderer, "Radeon") && (                                  // This looks like an AMD Radeon card...
-			(strstr(gl_vendor, "ATI") || strstr(gl_vendor, "AMD")) ||       // ...and AMD's official driver...
-			(strstr(gl_vendor, "Google") && strstr(gl_renderer, "OpenGL"))  // ...or ANGLE, backed by OpenGL.
-		)
-	) {
-		return "buggy AMD driver on macOS; see https://github.com/taisei-project/taisei/issues/182";
-	}
-#endif
-	return NULL;
-}
-
 static const char *detect_broken_norm16(void) {
 	const char *gl_vendor = get_unmasked_property(GL_VENDOR, true);
 	const char *gl_renderer = get_unmasked_property(GL_RENDERER, true);
@@ -813,12 +797,6 @@ static const char *detect_broken_norm16(void) {
 }
 
 static void glcommon_check_issues(void) {
-	glext.issues.avoid_sampler_uniform_updates = glcommon_check_workaround(
-		"avoid sampler uniform updates",
-		"TAISEI_GL_WORKAROUND_AVOID_SAMPLER_UNIFORM_UPDATES",
-		detect_slow_sampler_update
-	);
-
 	glext.issues.disable_norm16 = glcommon_check_workaround(
 		"disable normalized 16bpc pixel formats",
 		"TAISEI_GL_WORKAROUND_DISABLE_NORM16",
@@ -933,6 +911,7 @@ void glcommon_check_capabilities(void) {
 	glcommon_ext_float_blend();
 	glcommon_ext_instanced_arrays();
 	glcommon_ext_internalformat_query2();
+	glcommon_ext_invalidate_subdata();
 	glcommon_ext_pixel_buffer_object();
 	glcommon_ext_seamless_cubemap();
 	glcommon_ext_texture_filter_anisotropic();
@@ -942,6 +921,7 @@ void glcommon_check_capabilities(void) {
 	glcommon_ext_texture_half_float_linear();
 	glcommon_ext_texture_norm16();
 	glcommon_ext_texture_rg();
+	glcommon_ext_texture_storage();
 	glcommon_ext_texture_swizzle();
 	glcommon_ext_vertex_array_object();
 	glcommon_ext_viewport_array();
