@@ -2,28 +2,28 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
+#include "font.h"
+
+#include "config.h"
+#include "dynarray.h"
+#include "events.h"
+#include "memory/arena.h"
+#include "renderer/api.h"
+#include "util.h"
+#include "util/glm.h"
+#include "util/kvparser.h"
+#include "util/rectpack.h"
+#include "video.h"
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_STROKER_H
 #include FT_MODULE_H
 
-#include "font.h"
-#include "config.h"
-#include "events.h"
-#include "renderer/api.h"
-#include "util.h"
-#include "util/glm.h"
-#include "util/graphics.h"
-#include "util/rectpack.h"
-#include "video.h"
-#include "dynarray.h"
-
 static void init_fonts(void);
 static void post_init_fonts(void);
 static void shutdown_fonts(void);
@@ -129,6 +129,8 @@ static struct {
 	Texture *render_tex;
 	Framebuffer *render_buf;
 	SpriteSheetAnchor spritesheets;
+	MemArena arena;
+	RectPackSectionPool rpspool;
 
 	struct {
 		SDL_mutex *new_face;
@@ -192,6 +194,8 @@ static void *ft_realloc(FT_Memory mem, long cur_size, long new_size, void *block
 static void init_fonts(void) {
 	FT_Error err;
 
+	marena_init(&globals.arena, sizeof(RectPackSection) * 128);
+
 	static typeof(*(FT_Memory)0) ft_mem = {
 		.alloc = ft_alloc,
 		.free = ft_free,
@@ -253,6 +257,7 @@ static void shutdown_fonts(void) {
 	FT_Done_Library(globals.lib);
 	SDL_DestroyMutex(globals.mutex.new_face);
 	SDL_DestroyMutex(globals.mutex.done_face);
+	marena_deinit(&globals.arena);
 }
 
 static char *font_path(const char *name) {
@@ -394,6 +399,13 @@ void font_set_kerning_enabled(Font *font, bool newval) {
 #define SS_TEXTURE_TYPE TEX_TYPE_RGB_8
 #define SS_TEXTURE_FLAGS 0
 
+INLINE RectPackSectionSource rps_source(void) {
+	return (RectPackSectionSource) {
+		.arena = &globals.arena,
+		.pool = &globals.rpspool,
+	};
+}
+
 static SpriteSheet *add_spritesheet(SpriteSheetAnchor *spritesheets) {
 	auto ss = ALLOC(SpriteSheet, {
 		.tex = r_texture_create(&(TextureParams) {
@@ -405,12 +417,12 @@ static SpriteSheet *add_spritesheet(SpriteSheetAnchor *spritesheets) {
 			.filter.min = TEX_FILTER_LINEAR,
 			.wrap.s = TEX_WRAP_CLAMP,
 			.wrap.t = TEX_WRAP_CLAMP,
-			.mipmaps = TEX_MIPMAPS_MAX,
-			.mipmap_mode = TEX_MIPMAP_AUTO,
+			.mipmaps = 1,
+			.mipmap_mode = TEX_MIPMAP_MANUAL,
 		})
 	});
 
-	rectpack_init(&ss->rectpack, &default_allocator, SS_WIDTH, SS_HEIGHT);
+	rectpack_init(&ss->rectpack, SS_WIDTH, SS_HEIGHT);
 
 #ifdef DEBUG
 	char buf[128];
@@ -430,7 +442,7 @@ static bool add_glyph_to_spritesheet(Glyph *glyph, Pixmap *pixmap, SpriteSheet *
 	uint padded_w = pixmap->width + 2 * GLYPH_SPRITE_PADDING;
 	uint padded_h = pixmap->height + 2 * GLYPH_SPRITE_PADDING;
 
-	glyph->spritesheet_section = rectpack_add(&ss->rectpack, padded_w, padded_h);
+	glyph->spritesheet_section = rectpack_add(&ss->rectpack, rps_source(), padded_w, padded_h, false);
 
 	if(glyph->spritesheet_section == NULL) {
 		return false;
@@ -497,7 +509,7 @@ static const char *pixmode_name(FT_Pixel_Mode mode) {
 
 static void delete_spritesheet(SpriteSheetAnchor *spritesheets, SpriteSheet *ss) {
 	r_texture_destroy(ss->tex);
-	rectpack_deinit(&ss->rectpack);
+	assert(rectpack_is_empty(&ss->rectpack));
 	alist_unlink(spritesheets, ss);
 	mem_free(ss);
 }
@@ -739,7 +751,7 @@ static void wipe_glyph_cache(Font *font) {
 		RectPackSection *section = g->spritesheet_section;
 		assume(section != NULL);
 
-		rectpack_reclaim(rp, section);
+		rectpack_reclaim(rp, rps_source(), section);
 
 		if(rectpack_is_empty(rp)) {
 			delete_spritesheet(&globals.spritesheets, ss);
@@ -1364,8 +1376,7 @@ void text_render(const char *text, Font *font, Sprite *out_sprite, TextBBox *out
 	r_clear(BUFFER_COLOR, RGBA(0, 0, 0, 0), 1);
 
 	r_blend(BLEND_PREMUL_ALPHA);
-	r_enable(RCAP_CULL_FACE);
-	r_cull(CULL_BACK);
+	r_disable(RCAP_CULL_FACE);
 	r_disable(RCAP_DEPTH_TEST);
 
 	r_mat_mv_push_identity();
