@@ -2,16 +2,17 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
-
-#include "taisei.h"
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
 #include "basisu.h"
 #include "basisu_cache.h"
-#include "util/io.h"
+
 #include "rwops/rwops_sha256.h"
+#include "util.h"
+#include "util/env.h"
+#include "util/io.h"
 
 #include <basisu_transcoder_c_api.h>
 
@@ -361,8 +362,6 @@ uncompressed:
 		}
 
 		assert(qr.supplied_pixmap_format_supported);
-		assert(qr.supplied_pixmap_origin_supported);
-
 		return qr.optimal_pixmap_format;
 	}
 
