@@ -2,15 +2,17 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
-
-#include "taisei.h"
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
 #include "texture_loader.h"
 #include "basisu.h"
 
+#include "util.h"
+#include "util/io.h"
+#include "util/kvparser.h"
+
 void texture_loader_cleanup_stage1(TextureLoadData *ld) {
 	mem_free(ld->src_paths.main);
 	ld->src_paths.main = NULL;
@@ -643,6 +645,8 @@ void texture_loader_stage1(ResourceLoadState *st) {
 				}
 
 				break;
+
+			default: UNREACHABLE;
 		}
 
 		texture_loader_parse_filter(ld, "filter_min", str_filter_min, &ld->params.filter.min, true);
@@ -665,7 +669,7 @@ void texture_loader_stage1(ResourceLoadState *st) {
 			return;
 		}
 	} else {
-		ld->src_paths.main = strdup(st->path);
+		ld->src_paths.main = mem_strdup(st->path);
 	}
 
 
@@ -850,11 +854,22 @@ static void texture_loader_stage2(ResourceLoadState *st) {
 
 		texture = r_texture_create(&p);
 
+		if(!texture) {
+			texture_loader_failed(ld);
+			return;
+		}
+
 		char namebuf[strlen(st->name) + sizeof(" (transient)")];
 		snprintf(namebuf, sizeof(namebuf), "%s (transient)", st->name);
 		r_texture_set_debug_label(texture, namebuf);
 	} else {
 		texture = r_texture_create(&ld->params);
+
+		if(!texture) {
+			texture_loader_failed(ld);
+			return;
+		}
+
 		r_texture_set_debug_label(texture, st->name);
 	}
 
