@@ -2,34 +2,35 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
-#include "taisei.h"
-
-#include "util.h"
 #include "stage.h"
+
+#include "audio/audio.h"
+#include "common_tasks.h"  // IWYU pragma: keep
+#include "config.h"
+#include "dynstage.h"
+#include "eventloop/eventloop.h"
+#include "events.h"
 #include "global.h"
-#include "video.h"
-#include "resource/bgm.h"
+#include "lasers/draw.h"
+#include "log.h"
+#include "menu/gameovermenu.h"
+#include "menu/ingamemenu.h"
+#include "player.h"
 #include "replay/demoplayer.h"
 #include "replay/stage.h"
 #include "replay/state.h"
 #include "replay/struct.h"
-#include "config.h"
-#include "player.h"
-#include "menu/ingamemenu.h"
-#include "menu/gameovermenu.h"
-#include "audio/audio.h"
-#include "log.h"
-#include "stagetext.h"
+#include "resource/bgm.h"
 #include "stagedraw.h"
-#include "stageobjects.h"
-#include "eventloop/eventloop.h"
-#include "common_tasks.h"
 #include "stageinfo.h"
-#include "dynstage.h"
+#include "stageobjects.h"
+#include "stagetext.h"
+#include "util/env.h"
+#include "watchdog.h"
 
 typedef struct StageFrameState {
 	StageInfo *stage;
@@ -40,6 +41,7 @@ typedef struct StageFrameState {
 	bool quicksave_is_automatic;
 	bool quickload_requested;
 	bool was_skipping;
+	bool paused;
 	uint32_t dynstage_generation;
 	int transition_delay;
 	int desync_check_freq;
@@ -188,14 +190,28 @@ static bool ingame_menu_interrupts_bgm(void) {
 	return global.stage->type != STAGE_SPELL;
 }
 
-typedef struct IngameMenuContext {
+static inline bool is_quicksave_allowed(void) {
+#ifndef DEBUG
+	if(!global.is_practice_mode) {
+		return false;
+	}
+#endif
+
+	if(global.gameover != GAMEOVER_NONE) {
+		return false;
+	}
+
+	return true;
+}
+
+typedef struct IngameMenuCallContext {
 	CallChain next;
 	BGM *saved_bgm;
 	double saved_bgm_pos;
 	bool bgm_interrupted;
-} IngameMenuContext;
+} IngameMenuCallContext;
 
-static void setup_ingame_menu_bgm(IngameMenuContext *ctx, BGM *bgm) {
+static void setup_ingame_menu_bgm(IngameMenuCallContext *ctx, BGM *bgm) {
 	if(ingame_menu_interrupts_bgm()) {
 		ctx->bgm_interrupted = true;
 
@@ -209,7 +225,7 @@ static void setup_ingame_menu_bgm(IngameMenuContext *ctx, BGM *bgm) {
 	}
 }
 
-static void resume_bgm(IngameMenuContext *ctx) {
+static void resume_bgm(IngameMenuCallContext *ctx) {
 	if(ctx->bgm_interrupted) {
 		if(ctx->saved_bgm) {
 			audio_bgm_play(ctx->saved_bgm, true, ctx->saved_bgm_pos, 0.5);
@@ -223,7 +239,7 @@ static void resume_bgm(IngameMenuContext *ctx) {
 }
 
 static void stage_leave_ingame_menu(CallChainResult ccr) {
-	IngameMenuContext *ctx = ccr.ctx;
+	IngameMenuCallContext *ctx = ccr.ctx;
 	MenuData *m = ccr.result;
 
 	if(m->state != MS_Dead) {
@@ -249,13 +265,23 @@ static void stage_leave_ingame_menu(CallChainResult ccr) {
 
 static void stage_enter_ingame_menu(MenuData *m, BGM *bgm, CallChain next) {
 	events_emit(TE_GAME_PAUSE_STATE_CHANGED, true, NULL, NULL);
-	auto ctx = ALLOC(IngameMenuContext, { .next = next });
+	auto ctx = ALLOC(IngameMenuCallContext, { .next = next });
 	setup_ingame_menu_bgm(ctx, bgm);
 	pause_all_sfx();
 	enter_menu(m, CALLCHAIN(stage_leave_ingame_menu, ctx));
 }
 
-void stage_pause(void) {
+static void stage_unpause(CallChainResult ccr) {
+	StageFrameState *fstate = ccr.ctx;
+	fstate->paused = false;
+}
+
+static void stage_pause(StageFrameState *fstate) {
+	if(fstate->paused) {
+		log_debug("Pause request ignored, already paused");
+		return;
+	}
+
 	if(global.gameover == GAMEOVER_TRANSITIONING || stage_is_skip_mode()) {
 		return;
 	}
@@ -268,12 +294,55 @@ void stage_pause(void) {
 		m = create_ingame_menu();
 	}
 
-	stage_enter_ingame_menu(m, NULL, NO_CALLCHAIN);
+	fstate->paused = true;
+	stage_enter_ingame_menu(m, NULL, CALLCHAIN(stage_unpause, fstate));
+}
+
+static void stage_do_quickload(StageFrameState *fstate);
+
+static void stage_continue_game(void) {
+	log_info("The game is being continued...");
+	player_event(&global.plr, &global.replay.input, &global.replay.output, EV_CONTINUE, 0);
+}
+
+static void gameover_menu_result(CallChainResult ccr) {
+	GameoverMenuAction *_action = ccr.ctx;
+	auto action = *_action;
+	mem_free(_action);
+
+	switch(action) {
+		case GAMEOVERMENU_ACTION_QUIT:
+			if(global.plr.stats.total.continues_used >= MAX_CONTINUES) {
+				global.gameover = GAMEOVER_DEFEAT;
+			} else {
+				global.gameover = GAMEOVER_ABORT;
+			}
+			break;
+
+		case GAMEOVERMENU_ACTION_CONTINUE:
+			stage_continue_game();
+			break;
+
+		case GAMEOVERMENU_ACTION_RESTART:
+			global.gameover = GAMEOVER_RESTART;
+			break;
+
+		case GAMEOVERMENU_ACTION_QUICKLOAD:
+			stage_do_quickload(_current_stage_state);
+			break;
+
+		default: UNREACHABLE;
+	}
 }
 
 void stage_gameover(void) {
 	if(global.stage->type == STAGE_SPELL && config_get_int(CONFIG_SPELLSTAGE_AUTORESTART)) {
-		global.gameover = GAMEOVER_RESTART;
+		auto fstate = _current_stage_state;
+		if(fstate->quicksave) {
+			stage_do_quickload(fstate);
+		} else {
+			global.gameover = GAMEOVER_RESTART;
+		}
 		return;
 	}
 
@@ -284,10 +353,18 @@ void stage_gameover(void) {
 		progress_unlock_bgm("gameover");
 	}
 
-	stage_enter_ingame_menu(create_gameover_menu(), bgm, NO_CALLCHAIN);
+	auto action = ALLOC(GameoverMenuAction);
+	auto menu = create_gameover_menu(&(GameoverMenuParams) {
+		.output = action,
+		.quickload_shown = is_quicksave_allowed(),
+		.quickload_enabled = _current_stage_state->quicksave != NULL,
+	});
+
+	stage_enter_ingame_menu(menu, bgm, CALLCHAIN(gameover_menu_result, action));
 }
 
 static bool stage_input_common(SDL_Event *event, void *arg) {
+	StageFrameState *fstate = NOT_NULL(arg);
 	TaiseiEvent type = TAISEI_EVENT(event->type);
 	int32_t code = event->user.code;
 
@@ -306,7 +383,7 @@ static bool stage_input_common(SDL_Event *event, void *arg) {
 			break;
 
 		case TE_GAME_PAUSE:
-			stage_pause();
+			stage_pause(fstate);
 			break;
 
 		default:
@@ -377,20 +454,6 @@ static Replay *create_quicksave_replay(ReplayStage *rstg_src) {
 	return rpy;
 }
 
-static inline bool is_quicksave_allowed(void) {
-#ifndef DEBUG
-	if(!global.is_practice_mode) {
-		return false;
-	}
-#endif
-
-	if(global.gameover != GAMEOVER_NONE) {
-		return false;
-	}
-
-	return true;
-}
-
 static void stage_do_quicksave(StageFrameState *fstate, bool isauto) {
 	if(isauto && fstate->quicksave && !fstate->quicksave_is_automatic) {
 		// Do not overwrite a manual quicksave with an auto quicksave
@@ -522,10 +585,12 @@ static void leave_replay_mode(StageFrameState *fstate, ReplayState *rp_in) {
 static void replay_input(StageFrameState *fstate) {
 	if(!should_skip_frame(fstate)) {
 		events_poll((EventHandler[]){
-			{ .proc =
-				stage_is_demo_mode()
-					? stage_input_handler_demo
-					: stage_input_handler_replay
+			{
+				.proc =
+					stage_is_demo_mode()
+						? stage_input_handler_demo
+						: stage_input_handler_replay,
+				.arg = fstate,
 			},
 			{ NULL }
 		}, EFLAG_GAME);
@@ -671,6 +736,11 @@ static bool ellipse_predicate(EntityInterface *ent, void *varg) {
 
 void stage_clear_hazards_at(cmplx origin, double radius, ClearHazardsFlags flags) {
 	Circle area = { origin, radius };
+
+	if(UNLIKELY(radius <= 0)) {
+		return;
+	}
+
 	stage_clear_hazards_predicate(proximity_predicate, &area, flags);
 }
 
@@ -884,6 +954,10 @@ static LogicFrameAction stage_logic_frame(void *arg) {
 
 	stage_update_fps(fstate);
 
+	if(watchdog_signaled() && !stage_is_demo_mode()) {
+		global.gameover = GAMEOVER_ABORT;
+	}
+
 	if(stage_is_skip_mode()) {
 		global.plr.iddqd = true;
 	}
@@ -1011,6 +1085,14 @@ static void stage_end_loop(void *ctx);
 static void stage_stub_proc(void) { }
 static void stage_preload_stub_proc(ResourceGroup *rg) { }
 
+static void process_input(StageFrameState *fstate) {
+	if(global.replay.input.replay != NULL) {
+		replay_input(fstate);
+	} else {
+		stage_input(fstate);
+	}
+}
+
 TASK(stage_comain, { StageFrameState *fstate; }) {
 	StageFrameState *fstate = ARGS.fstate;
 	StageInfo *stage = fstate->stage;
@@ -1022,13 +1104,12 @@ TASK(stage_comain, { StageFrameState *fstate; }) {
 		display_stage_title(stage);
 	}
 
-	for(;;YIELD) {
-		if(global.replay.input.replay != NULL) {
-			replay_input(fstate);
-		} else {
-			stage_input(fstate);
-		}
+	YIELD;
 
+	bool first_frame = true;
+
+	for(;;) {
+		process_input(fstate);
 		process_boss(&global.boss);
 		process_enemies(&global.enemies);
 		process_projectiles(&global.projs, true);
@@ -1053,6 +1134,13 @@ TASK(stage_comain, { StageFrameState *fstate; }) {
 				ent_damage(&global.boss->ent, &(DamageInfo) { 400, DMG_PLAYER_SHOT } );
 			}
 		}
+
+		if(first_frame) {
+			// HACK: Run frame 0 logic twice for compatibility with a v1.4 bug
+			first_frame = false;
+		} else {
+			YIELD;
+		}
 	}
 }
 
