@@ -2,23 +2,19 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
-*/
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
+ */
 
-#include "taisei.h"
+#include "timeline.h"   // IWYU pragma: keep
 
-#include "timeline.h"
-#include "wriggle.h"
-#include "hina.h"
-#include "nonspells/nonspells.h"
-#include "stage2.h"
 #include "background_anim.h"
+#include "hina.h"
+#include "nonspells/nonspells.h"   // IWYU pragma: keep
+#include "stage2.h"
+#include "wriggle.h"
 
-#include "stage.h"
-#include "global.h"
-#include "common_tasks.h"
-#include "enemy_classes.h"
+#include "stages/common_imports.h"
 
 TASK(spinshot_fairy_attack_spawn_projs, {
 	BoxedEnemy e;
