@@ -2,8 +2,8 @@
  * This software is licensed under the terms of the MIT License.
  * See COPYING for further information.
  * ---
- * Copyright (c) 2011-2019, Lukas Weber <laochailan@web.de>.
- * Copyright (c) 2012-2019, Andrei Alexeyev <akari@taisei-project.org>.
+ * Copyright (c) 2011-2024, Lukas Weber <laochailan@web.de>.
+ * Copyright (c) 2012-2024, Andrei Alexeyev <akari@taisei-project.org>.
  */
 
 #pragma once
@@ -16,7 +16,7 @@ DIAGNOSTIC_CLANG(ignored "-Wnullability-completeness")
 #include <zip.h>
 DIAGNOSTIC_CLANG(pop)
 
-#include "util/libzip_compat.h"
+#include "util/libzip_compat.h"   // IWYU pragma: export
 
 #include "private.h"
 #include "hashtable.h"
